// Prototypes de d3dmisc.cpp
HRESULT hrPreInitD3D(void);
HRESULT hrInitD3D(HWND hWnd, const GUID* pDeviceGUID);
HRESULT hrCloseD3D(BOOL bFull);
HRESULT hrShowFrame(void);
HRESULT hrRestoreSurfaces(void);
VOID    vOnMove(int x, int y);
HRESULT hrRender3DEnvironment(void);

// Prototypes de Render.cpp
void    vSetD3DState(void);
HRESULT hrInitWorld( LPDIRECT3DDEVICE7 pd3dDevice );
HRESULT hrRender( LPDIRECT3DDEVICE7 pd3dDevice );

// Prototypes de menuwproc.cpp
LRESULT CALLBACK lrMenuWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
void    vUpdateMenu(void);
void    vRemakeTriview(LPARAM);

// Prototypes de 3dwproc.cpp
LRESULT CALLBACK lrPerspWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
void    vForce3DRefresh(BOOL bFull);

// Prototypes de 2dwproc.cpp
LRESULT CALLBACK lrPlanarWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
void    vForce2DRefresh(BOOL bFull);

// Prototypes de logo.cpp
void    vLogo(void);

// Prototypes de winstuff.c
void    vTrace(char *Str, ...);
BOOL    bPrepareGadgets(void);
void    vEraseGadgets(void);

// Prototypes de file-open-sculpt.cpp
void    vLoadSculpt(void);
void    vSaveSculpt(void);

// Prototypes de file-bin.cpp
void    vSaveBin(void);
void    vLoadBin(void);

// Prototypes de file-directx.cpp
void    vSaveDirectX(void);
void    vLoadDirectX(void);

// prototypes de dialogs.cpp
BOOL CALLBACK bGlobLightDlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK bSphereDlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK bFaceDlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK bExpandDlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK bHelixDlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK bHoleDlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK bStringDlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

// Prototypes de BaseObjects.cpp
BOOL    bSameMaterial(D3DMATERIAL7 *hMtrl1, D3DMATERIAL7 *hMtrl2);
int     iFindMaterial(D3DMATERIAL7 *hMtrl, int iPrevious);
BOOL    bTextureMaterial(int iMtrl, char *sTexName);
int     iFastMaterial(char *sName, unsigned char bRouge, unsigned char bVert, unsigned char bBleu);
int     iMakeMaterial(D3DMATERIAL7 *hMtrl, char *sName, BOOL bSame);
BOOL    bDeleteMaterial(int iMtrl);

int     iFindVertex(D3DVECTOR p, int iPrevious);
int     iMakeVertex(D3DVECTOR p, BOOL bSame);
BOOL    bVertInEdge(int iVert, int iEdge);
BOOL    bVertInTriangle(int iVert, int iTriangle);
BOOL    bIsVertexSelected(int iVertex);
int     iEdgesPerVert(int iVert);
int     iTrianglesPerVert(int iVert);
BOOL    bDeleteVertex(int iCnt);
#ifndef _AMIGA_
BOOL    bSmoothNorm(int iVertex);
#endif

int     iFindEdge(int i1, int i2);
int     iMakeEdge(int i1, int i2);
BOOL    bDeleteEdge(int iCnt);
BOOL    bIsEdgeSelected(int iTriangle);
BOOL    bIsEdgePartiallySelected(int iTriangle);

int     iFindTriangle(int i1, int i2, int i3);
int     iMakeTriangle(int i1, int i2, int i3, int iMtrl); // unsigned char bRouge, unsigned char bVert, unsigned char bBleu);
BOOL    bSetUV(int iTriangle, int u1, int v1, int u2, int v2, int u3, int v3);
BOOL    bDeleteTriangle(int iCnt);
#ifndef _AMIGA_
BOOL    bUpdateD3DTri(int iCnt);
#endif

BOOL    bSubdivideTriangle(int iTriangle);
BOOL    bIsTriangleSelected(int iTriangle);
BOOL    bIsTrianglePartiallySelected(int iTriangle);

int iFindLamp(D3DVECTOR p, int iPrevious);
int iMakeLamp(
    D3DLIGHTTYPE    dltType,            /* Type of light source */
    D3DCOLORVALUE   dcvDiffuse,         /* Diffuse color of light */
    D3DCOLORVALUE   dcvSpecular,        /* Specular color of light */
    D3DCOLORVALUE   dcvAmbient,         /* Ambient color of light */
    D3DVECTOR       dvPosition,         /* Position in world space */
    D3DVECTOR       dvDirection,        /* Direction in world space */
    D3DVALUE        dvRange,            /* Cutoff range */
    D3DVALUE        dvFalloff,          /* Falloff */
    D3DVALUE        dvAttenuation0,     /* Constant attenuation */
    D3DVALUE        dvAttenuation1,     /* Linear attenuation */
    D3DVALUE        dvAttenuation2,     /* Quadratic attenuation */
    D3DVALUE        dvTheta,            /* Inner angle of spotlight cone */
    D3DVALUE        dvPhi               /* Outer angle of spotlight cone */
);
BOOL    bUpdateLamp(int iLamp);
BOOL    bDeleteLamp(int iLamp);

void    vResetProcess(void);
void    vDeleteObjects(void);
void    vPropagateSelection();
BOOL    bCloneSelection(D3DVECTOR vShift, BOOL bCloneMode);
void    vHideSelection(void);
void    vRevealHidden(void);
D3DVECTOR vCenter(void);
void    vCollect(void);

// prototypes de GeomObjects.cpp
void    vAddSphere(D3DVECTOR vCenter, float r, int horiz, int vert, BOOL bHalf);
void    vAddBoing(D3DVECTOR vCenter, float r);
void    vAddTorus(D3DVECTOR vCenter, float R, float r, int n, int m);
void    vAddDisk(D3DVECTOR vCenter, float r, int iSubdiv);
void    vAddCircle(D3DVECTOR vCenter, float r, int iSubdiv);

// Prototypes de Objects - D3D
void    vDeleteD3DObjects(void);
int     iMakeD3DObj(void);
int     iFindD3DObject(D3DVECTOR p, int iPrevious);
BOOL    bMoveD3DObj(int iObject, D3DVECTOR vPos);

// Prototypes de d3denum.cpp
HRESULT D3DEnum_SelectDefaultDevice(D3DEnum_DeviceInfo** ppDevice, DWORD dwFlags);
HRESULT D3DEnum_UserChangeDevice(D3DEnum_DeviceInfo** ppDevice);

// Prototypes de triangulate.cpp
void    fill(void);

// Prototypes de triangulator.cpp
void    triangulate(char *triswitches, struct triangulateio *in, struct triangulateio *out, struct triangulateio *vorout);

// Prototypes de FontOutliner.cpp
void    vAddTextOutline(void);

#ifdef _AMIGA_
// Prototypes de asl.c
int FSInit(void);
int FSelect(char *Bouton, char *Fichier);
void FSClose(void);
int SMInit(void);
int SMSelect(void);
void SMClose(void);

// Prototypes de console.cpp
BOOL InitConsole(HWND);
void CloseConsole(void);
void ConPutS(char *);

// Prototypes de amistuff.cpp spcifiques Amiga (les protos des fonctions Windows sont dans le fake windows.h)
HGDIOBJ hAddImage(HWND hWnd, char *sName, int iX, int iY, int iWidth, int iHeight);
void vRefreshImage(HWND hWnd, HGDIOBJ hObject);
void vMoveImage(HWND hWnd, HGDIOBJ hObject, int iX, int iY);
void vRemoveImage(HWND hWnd, HGDIOBJ hObject);
void UnclipWindow(HWND);
struct Region *ClipWindow(HWND, LONG, LONG, LONG, LONG);

// Prototypes de resload.cpp spcifiques Amiga (les protos des fonctions Windows sont dans le fake windows.h)
BOOL DLGInit(char *sResFileName);
void DLGClose(void);

// Prototype de CloseWindowSafely.cpp
void CloseWindowSafely(struct Window *win);

// prototypes de ami-texture.cpp
void vInitTextures(W3D_Context *hW3DC);
void vCloseTextures(W3D_Context *hW3DC);

// prototypes de vDemo.cpp
void vDemo(void);
#endif
